@@ -1,5 +1,6 @@
 use crate::ProfileIdentifier;
-use ockam_core::{Decodable, Message, Result, Routed};
+use ockam_core::{async_trait, compat::boxed::Box};
+use ockam_core::{AccessControl, LocalMessage, Result};
 
 mod secure_channel_worker;
 pub(crate) use secure_channel_worker::*;
@@ -12,36 +13,54 @@ pub use trust_policy::*;
 mod local_info;
 pub use local_info::*;
 
-// TODO: rename
-pub fn check_message_origin<T: Message>(
-    msg: &Routed<T>,
-    their_profile_id: &ProfileIdentifier,
-) -> Result<bool> {
-    let local_msg = msg.local_message();
-    let local_info = LocalInfo::decode(local_msg.local_info())?;
+pub struct EntityAccessControlBuilder;
 
-    let res = local_info.their_profile_id() == their_profile_id;
+impl EntityAccessControlBuilder {
+    pub fn new_with_id(their_profile_id: ProfileIdentifier) -> EntityIdAccessControl {
+        EntityIdAccessControl { their_profile_id }
+    }
 
-    Ok(res)
+    pub fn new_with_any_id() -> EntityAnyIdAccessControl {
+        EntityAnyIdAccessControl
+    }
 }
 
-// TODO: rename
-pub fn get_secure_channel_participant_id<T: Message>(msg: &Routed<T>) -> Result<ProfileIdentifier> {
-    let local_msg = msg.local_message();
-    let local_info = LocalInfo::decode(local_msg.local_info())?;
+pub struct EntityAnyIdAccessControl;
 
-    let res = local_info.their_profile_id().clone();
+#[async_trait]
+impl AccessControl for EntityAnyIdAccessControl {
+    async fn msg_is_authorized(&mut self, local_msg: &LocalMessage) -> Result<bool> {
+        Ok(EntitySecureChannelLocalInfo::find_info(local_msg).is_ok())
+    }
+}
 
-    Ok(res)
+pub struct EntityIdAccessControl {
+    their_profile_id: ProfileIdentifier,
+}
+
+#[async_trait]
+impl AccessControl for EntityIdAccessControl {
+    async fn msg_is_authorized(&mut self, local_msg: &LocalMessage) -> Result<bool> {
+        if let Ok(msg_profile_id) = EntitySecureChannelLocalInfo::find_info(local_msg) {
+            Ok(msg_profile_id.their_profile_id() == &self.their_profile_id)
+        } else {
+            Ok(false)
+        }
+    }
 }
 
 #[cfg(test)]
 mod test {
     use super::*;
     use crate::{Entity, Identity};
-    use ockam_core::{route, Route};
+    use core::sync::atomic::{AtomicU8, Ordering};
+    use ockam_core::compat::sync::Arc;
+    use ockam_core::{route, Any, Route, Routed, Worker};
     use ockam_node::Context;
     use ockam_vault_sync_core::Vault;
+    use std::convert::TryInto;
+    use std::time::Duration;
+    use tokio::time::sleep;
 
     #[ockam_macros::test]
     async fn test_channel(ctx: &mut Context) -> Result<()> {
@@ -68,7 +87,7 @@ mod test {
         .await?;
         let msg = ctx.receive::<String>().await?.take();
 
-        let local_info = LocalInfo::decode(msg.local_message().local_info())?;
+        let local_info = EntitySecureChannelLocalInfo::find_info(msg.local_message())?;
         assert_eq!(local_info.their_profile_id(), &alice.identifier().await?);
 
         let return_route = msg.return_route();
@@ -78,9 +97,7 @@ mod test {
 
         let msg = ctx.receive::<String>().await?.take();
 
-        let local_info = msg.local_message().local_info();
-
-        let local_info = LocalInfo::decode(local_info)?;
+        let local_info = EntitySecureChannelLocalInfo::find_info(msg.local_message())?;
         assert_eq!(local_info.their_profile_id(), &bob.identifier().await?);
 
         assert_eq!("Hello, Alice!", msg.body());
@@ -232,4 +249,122 @@ mod test {
 
         ctx.stop().await
     }
+
+    struct Receiver {
+        received_count: Arc<AtomicU8>,
+    }
+
+    #[ockam_core::async_trait]
+    impl Worker for Receiver {
+        type Message = Any;
+        type Context = Context;
+
+        async fn handle_message(
+            &mut self,
+            _context: &mut Self::Context,
+            _msg: Routed<Self::Message>,
+        ) -> Result<()> {
+            self.received_count.fetch_add(1, Ordering::Relaxed);
+
+            Ok(())
+        }
+    }
+
+    #[allow(non_snake_case)]
+    #[ockam_macros::test]
+    async fn access_control__known_participant__should_pass_messages(
+        ctx: &mut Context,
+    ) -> Result<()> {
+        let received_count = Arc::new(AtomicU8::new(0));
+        let receiver = Receiver {
+            received_count: received_count.clone(),
+        };
+
+        let vault = Vault::create(ctx).await?;
+
+        let mut alice = Entity::create(ctx, &vault).await?;
+        let mut bob = Entity::create(ctx, &vault).await?;
+
+        let access_control = EntityAccessControlBuilder::new_with_id(alice.identifier().await?);
+        ctx.start_worker_with_access_control("receiver", receiver, access_control)
+            .await?;
+
+        bob.create_secure_channel_listener("listener", TrustEveryonePolicy)
+            .await?;
+
+        let alice_channel = alice
+            .create_secure_channel("listener", TrustEveryonePolicy)
+            .await?;
+
+        ctx.send(route![alice_channel, "receiver"], "Hello, Bob!".to_string())
+            .await?;
+
+        sleep(Duration::from_secs(1)).await;
+
+        assert_eq!(received_count.load(Ordering::Relaxed), 1);
+
+        ctx.stop().await
+    }
+
+    #[allow(non_snake_case)]
+    #[ockam_macros::test]
+    async fn access_control__unknown_participant__should_not_pass_messages(
+        ctx: &mut Context,
+    ) -> Result<()> {
+        let received_count = Arc::new(AtomicU8::new(0));
+        let receiver = Receiver {
+            received_count: received_count.clone(),
+        };
+
+        let vault = Vault::create(ctx).await?;
+
+        let mut alice = Entity::create(ctx, &vault).await?;
+        let mut bob = Entity::create(ctx, &vault).await?;
+
+        let access_control = EntityAccessControlBuilder::new_with_id(bob.identifier().await?);
+        ctx.start_worker_with_access_control("receiver", receiver, access_control)
+            .await?;
+
+        bob.create_secure_channel_listener("listener", TrustEveryonePolicy)
+            .await?;
+
+        let alice_channel = alice
+            .create_secure_channel("listener", TrustEveryonePolicy)
+            .await?;
+
+        ctx.send(route![alice_channel, "receiver"], "Hello, Bob!".to_string())
+            .await?;
+
+        sleep(Duration::from_secs(1)).await;
+
+        assert_eq!(received_count.load(Ordering::Relaxed), 0);
+
+        ctx.stop().await
+    }
+
+    #[allow(non_snake_case)]
+    #[ockam_macros::test]
+    async fn access_control__no_secure_channel__should_not_pass_messages(
+        ctx: &mut Context,
+    ) -> Result<()> {
+        let received_count = Arc::new(AtomicU8::new(0));
+        let receiver = Receiver {
+            received_count: received_count.clone(),
+        };
+
+        let access_control = EntityAccessControlBuilder::new_with_id(
+            "P79b26ba2ea5ad9b54abe5bebbcce7c446beda8c948afc0de293250090e5270b6".try_into()?,
+        );
+        ctx.start_worker_with_access_control("receiver", receiver, access_control)
+            .await?;
+
+        ctx.send(route!["receiver"], "Hello, Bob!".to_string())
+            .await?;
+
+        sleep(Duration::from_secs(1)).await;
+
+        assert_eq!(received_count.load(Ordering::Relaxed), 0);
+
+        ctx.stop().await
+    }
 }
