@@ -3,14 +3,13 @@ use crate::{
     CredentialAttributeType, CredentialError, CredentialFragment1, CredentialFragment2,
     CredentialHolder, CredentialIssuer, CredentialOffer, CredentialPresentation, CredentialRequest,
     CredentialSchema, CredentialVerifier, EntityCredential, EntityError, ExtPokSignatureProof,
-    Identity, OfferId, PresentationManifest, ProfileState, ProofBytes, ProofRequestId,
-    SigningPublicKey,
+    OfferId, PresentationManifest, ProfileState, ProofBytes, ProofRequestId, SigningPublicKey,
 };
 use core::convert::TryInto;
 use ockam_core::compat::collections::{HashMap, HashSet};
+use ockam_core::vault::SecretVault;
 use ockam_core::{allow, deny, Result};
 use ockam_core::{async_trait, compat::boxed::Box};
-use ockam_vault_core::SecretVault;
 use rand::thread_rng;
 use sha2::digest::{generic_array::GenericArray, Digest, FixedOutput};
 use signature_bbs_plus::{Issuer as BbsIssuer, PokSignatureProof, Prover};
