@@ -43,7 +43,7 @@ impl rand::RngCore for MockRng {
 }
 
 #[allow(clippy::ptr_arg)]
-pub fn write_attributes<S>(v: &Vec<CredentialAttributeSchema>, s: S) -> Result<S::Ok, S::Error>
+pub fn write_attributes<S>(v: &[CredentialAttributeSchema], s: S) -> Result<S::Ok, S::Error>
 where
     S: Serializer,
 {
@@ -77,7 +77,7 @@ where
             let mut buf = Vec::new();
             while let Some(a) = s.next_element()? {
                 let _result = buf.push(a);
-                #[cfg(all(not(feature = "std"), not(feature = "alloc")))]
+                #[cfg(not(feature = "alloc"))]
                 {
                     _result.map_err(|_| DError::invalid_length(_l, &self))?;
                 }
