@@ -8,10 +8,10 @@ use ockam_core::compat::{
     string::{String, ToString},
     vec::Vec,
 };
+use ockam_core::vault::{PublicKey, Secret};
 use ockam_core::{async_trait, compat::boxed::Box};
 use ockam_core::{Address, AsyncTryClone, Result, Route};
 use ockam_node::{Context, Handle};
-use ockam_vault::ockam_vault_core::{PublicKey, Secret};
 use IdentityRequest::*;
 use IdentityResponse as Res;
 #[derive(AsyncTryClone)]
@@ -92,11 +92,19 @@ impl Identity for Entity {
         self.cast(CreateKey(self.id(), label)).await
     }
 
-    async fn rotate_profile_key(&mut self) -> Result<()> {
+    async fn add_key(&mut self, label: String, secret: &Secret) -> Result<()> {
+        if let Res::AddKey = self.call(AddKey(self.id(), label, secret.clone())).await? {
+            Ok(())
+        } else {
+            err()
+        }
+    }
+
+    async fn rotate_root_secret_key(&mut self) -> Result<()> {
         self.cast(RotateKey(self.id())).await
     }
 
-    async fn get_profile_secret_key(&self) -> Result<Secret> {
+    async fn get_root_secret_key(&self) -> Result<Secret> {
         if let Res::GetProfileSecretKey(secret) = self.call(GetProfileSecretKey(self.id())).await? {
             Ok(secret)
         } else {
@@ -112,7 +120,7 @@ impl Identity for Entity {
         }
     }
 
-    async fn get_profile_public_key(&self) -> Result<PublicKey> {
+    async fn get_root_public_key(&self) -> Result<PublicKey> {
         if let Res::GetProfilePublicKey(public_key) =
             self.call(GetProfilePublicKey(self.id())).await?
         {
