@@ -1,21 +1,21 @@
 use crate::change_history::ProfileChangeHistory;
 use crate::profile::Profile;
 use crate::{
-    ChangeSet, EntityError, EventIdentifier, KeyAttributes, MetaKeyAttributes, ProfileChange,
-    ProfileChangeEvent, ProfileChangeProof, ProfileChangeType, ProfileEventAttributes,
-    ProfileState, Signature, SignatureType,
+    ChangeBlock, EntityError, EventIdentifier, KeyAttributes, MetaKeyAttributes, ProfileChange,
+    ProfileChangeEvent, ProfileChangeType, ProfileEventAttributes, ProfileState, Signature,
+    SignatureType,
 };
-use ockam_core::compat::vec::Vec;
-use ockam_core::Encodable;
-use ockam_vault::ockam_vault_core::Signature as OckamVaultSignature;
-use ockam_vault::ockam_vault_core::{Hasher, SecretVault, Signer};
+use ockam_core::vault::PublicKey;
+use ockam_core::vault::Signature as OckamVaultSignature;
+use ockam_core::vault::{Hasher, SecretVault, Signer};
+use ockam_core::{Encodable, Result};
 use serde::{Deserialize, Serialize};
 
 /// RotateKeyChangeData
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub struct RotateKeyChangeData {
     key_attributes: KeyAttributes,
-    public_key: Vec<u8>,
+    public_key: PublicKey,
 }
 
 impl RotateKeyChangeData {
@@ -24,14 +24,14 @@ impl RotateKeyChangeData {
         &self.key_attributes
     }
     /// Return public key
-    pub fn public_key(&self) -> &[u8] {
-        self.public_key.as_slice()
+    pub fn public_key(&self) -> &PublicKey {
+        &self.public_key
     }
 }
 
 impl RotateKeyChangeData {
     /// Create RotateKeyChangeData
-    pub fn new(key_attributes: KeyAttributes, public_key: Vec<u8>) -> Self {
+    pub fn new(key_attributes: KeyAttributes, public_key: PublicKey) -> Self {
         RotateKeyChangeData {
             key_attributes,
             public_key,
@@ -79,35 +79,28 @@ impl RotateKeyChange {
 
 impl ProfileState {
     /// Rotate key event
-    pub(crate) async fn rotate_key(
+    pub(crate) async fn make_rotate_key_event(
         &mut self,
         key_attributes: KeyAttributes,
         attributes: ProfileEventAttributes,
-    ) -> ockam_core::Result<ProfileChangeEvent> {
+    ) -> Result<ProfileChangeEvent> {
         let prev_event_id = self.change_history().get_last_event_id()?;
 
         let last_event_in_chain = ProfileChangeHistory::find_last_key_event(
             self.change_history().as_ref(),
-            &key_attributes,
+            key_attributes.label(),
         )?
         .clone();
 
         let last_key_in_chain =
-            Self::get_secret_key_from_event(&key_attributes, &last_event_in_chain, &mut self.vault)
-                .await?;
+            Self::get_secret_key_from_event(&last_event_in_chain, &mut self.vault).await?;
 
         let secret_attributes = match key_attributes.meta() {
             MetaKeyAttributes::SecretAttributes(secret_attributes) => *secret_attributes,
-            _ => panic!("missing secret attributes"),
         };
 
         let secret_key = self.vault.secret_generate(secret_attributes).await?;
-        let public_key = self
-            .vault
-            .secret_public_key_get(&secret_key)
-            .await?
-            .as_ref()
-            .to_vec();
+        let public_key = self.vault.secret_public_key_get(&secret_key).await?;
 
         let data = RotateKeyChangeData::new(key_attributes, public_key);
         let data_binary = data.encode().map_err(|_| EntityError::BareError)?;
@@ -121,19 +114,22 @@ impl ProfileState {
             attributes,
             ProfileChangeType::RotateKey(change),
         );
-        let changes = ChangeSet::new(prev_event_id, vec![profile_change]);
-        let changes_binary = changes.encode().map_err(|_| EntityError::BareError)?;
+        let change_block = ChangeBlock::new(prev_event_id, profile_change);
+        let change_block_binary = change_block.encode().map_err(|_| EntityError::BareError)?;
 
-        let event_id = self.vault.sha256(&changes_binary).await?;
+        let event_id = self.vault.sha256(&change_block_binary).await?;
         let event_id = EventIdentifier::from_hash(event_id);
 
-        let root_key = self.get_root_secret().await?;
+        let self_signature = self.vault.sign(&secret_key, event_id.as_ref()).await?;
+        let self_signature = Signature::new(SignatureType::SelfSign, self_signature);
 
-        let signature = self.vault.sign(&root_key, event_id.as_ref()).await?;
+        let root_key = self.get_root_secret_key().await?;
 
-        let proof =
-            ProfileChangeProof::Signature(Signature::new(SignatureType::RootSign, signature));
-        let signed_change_event = ProfileChangeEvent::new(event_id, changes, proof);
+        let root_signature = self.vault.sign(&root_key, event_id.as_ref()).await?;
+        let root_signature = Signature::new(SignatureType::RootSign, root_signature);
+
+        let signed_change_event =
+            ProfileChangeEvent::new(event_id, change_block, vec![self_signature, root_signature]);
 
         Ok(signed_change_event)
     }
